# -*- coding: utf-8 -*-
import logging
from os import listdir

import click
import scrapy
import jsonlines
from scrapy.exceptions import CloseSpider
from scrapy.crawler import CrawlerProcess
from geo_alchemy import PlatformParser, SampleParser, SeriesParser


logger = logging.getLogger('geo-spider')
modes = ['incremental', 'missed']
settings = {
    'USER_AGENT': (
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit'
        '/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36'
    )
}


class PlatformItem(scrapy.Item):
    title = scrapy.Field()
    accession = scrapy.Field()
    technology = scrapy.Field()
    distribution = scrapy.Field()
    organisms = scrapy.Field()
    manufacturer = scrapy.Field()
    manufacturer_protocol = scrapy.Field()
    description = scrapy.Field()
    columns = scrapy.Field()
    internal_data = scrapy.Field()
    release_date = scrapy.Field()
    last_update_date = scrapy.Field()
    submission_date = scrapy.Field()

    def __repr__(self):
        return 'PlatformItem<%s>' % self['accession']


class PlatformSpider(scrapy.Spider):
    name = 'platforms'

    def __init__(self, mode='incremental', **kwargs):
        assert mode in modes
        self.mode = mode
        super(PlatformSpider, self).__init__(**kwargs)

    def start_requests(self):
        url = 'https://www.ncbi.nlm.nih.gov/geo/browse/?view=platforms&display=20&zsort=date'
        yield scrapy.Request(url=url, callback=self.parse)

    def parse(self, response):
        for accession in response.xpath(
            '//table[@id="geo_data"]/tbody/tr/td[1]/a/text()'
        ).extract():
            if accession in PlatformParser.platforms:
                if self.mode == 'incremental':
                    raise CloseSpider('All platforms have crawled.')
                else:
                    continue
            logger.info('New accession found %s', accession)
            miniml_url = f'https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc={accession}&targ=self&form=xml&view=quick'
            yield scrapy.Request(miniml_url, callback=self.parse_platform)
        next_page = response.xpath('//div[@class="pager"]/span[@class="next"]/a/@href').get()
        if next_page:
            yield response.follow(next_page, callback=self.parse)

    def parse_platform(self, response):
        try:
            platform = PlatformParser.from_miniml(response.body).parse()
            yield PlatformItem(platform.to_dict())
        except Exception as exc:
            logger.error('Error occurred when parsing data from %s', response.url)
            logger.exception(exc)


@click.group()
@click.option(
    '-d', '--debug-mode', is_flag=True,
    help='enable debug mode'
)
@click.option(
    '-l', '--log-file', type=click.Path(exists=False),
    default='geo-spider.log', show_default=True, help='log file'
)
def geo_spider(debug_mode, log_file):
    """
    geo-spider
    """
    level = logging.DEBUG if debug_mode else logging.WARNING
    logging.basicConfig(level=level, filename=log_file)


@geo_spider.command(name='platforms')
@click.option(
    '-cf', '--crawled-file', required=False,
    type=click.Path(exists=True), help='crawled jsonlines file'
)
@click.option(
    '-cd', '--crawled-dir', required=False,
    type=click.Path(exists=True),
    help='crawled directory, a directory that contains all crawled jsonlines files.'
)
@click.option(
    '-m', '--mode', default='incremental', show_default=True,
    type=click.Choice(modes), help='crawling mode'
)
@click.option(
    '-o', '--outfile', required=False, show_default=True,
    default='platforms.jl', help='output jsonlines file'
)
def geo_spider_platforms(crawled_file, crawled_dir, mode, outfile):
    """
    crawl metadata of GEO platforms
    """
    try:
        # 1. read all crawled platform accessions
        if crawled_file:
            with jsonlines.open(crawled_file, mode='r') as reader:
                for data in reader:
                    PlatformParser.parse_dict(data)
        if crawled_dir:
            for crawled_file in listdir(crawled_dir):
                with jsonlines.open(crawled_file, mode='r') as reader:
                    for data in reader:
                        PlatformParser.parse_dict(data)
        if PlatformParser.platforms:
            print('%s crawled platforms totally.' % len(PlatformParser.platforms))
        else:
            print('No crawled platforms provided.')

        # 2. crawl new platforms
        settings['FEEDS'] = {outfile: {'format': 'jsonlines', 'encoding': 'utf-8'}}
        process = CrawlerProcess(settings)
        process.crawl(PlatformSpider, mode=mode)
        process.start()
    except jsonlines.jsonlines.InvalidLineError as exc:
        logging.exception(exc)
        raise click.UsageError('Invalid jsonlines file.')
    except Exception as exc:
        logging.exception(exc)
        raise click.UsageError('Unknown error occurred, refer to log file for details.')


if __name__ == '__main__':
    geo_spider()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import io
import os

from setuptools import setup

# Package meta-data.
S = 'soultoolman'
NAME = 'geo-spider'
DESCRIPTION = 'Crawling all GEO metadata.'
URL = 'https://github.com/soultoolman/geo-spider'
EMAIL = 'soultooman@gmail.com'
AUTHOR = S
REQUIRES_PYTHON = '>=3.6.0'
VERSION = '0.0.3'

# What packages are required for this module to be executed?
REQUIRED = [
    'click',
    'scrapy',
    'jsonlines',
    'geo-alchemy',
]

# What packages are optional?
EXTRAS = {
}


# What packages are required for testing
TESTS_REQUIRE = [
    'pytest'
]


# console scripts
CONSOLE_SCRIPTS = [
    'geo-spider=geo_spider:geo_spider'
]

# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the Trove Classifier
# for that!

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION

# Where the magic happens:
setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    # If your package is a single module, use this instead of 'packages':
    py_modules=['geo_spider'],

    entry_points={
        'console_scripts': CONSOLE_SCRIPTS
    },
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    tests_require=TESTS_REQUIRE,
    include_package_data=True,
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
    ],
    # $ setup.py publish support.
    cmdclass={
    },
)
